#include <iostream>
#include <iomanip>
#include <fstream>
#include <cmath>
#include <stdio.h>
#include <stdlib.h>
#include <string>

using namespace std;

void setColor(int idischarge, int& red, int& green, int& blue);

int main(void)
{
	cout << "Reading parameters for runoff simulation." << endl;
	ifstream finParam("../param-frm-event-ver300.dat");
	if(!finParam){
		cout << "param-frm-event-ver300.dat cannot open." << endl;
		cout << endl;
		cout << "Program ends. Hit any key!" << endl;
		char ch;
		cin.get(ch);
		return 1;
	}

	// vZ̈ simulation range
	cout << "1) Reading parameters for study area." << endl;

	float wTopoScale;

	finParam >> wTopoScale;
	cout << "  Topography scale : " << wTopoScale << endl;

	int wCol,wRow;
	double wSWLON, wSWLAT;

	finParam >> wCol; 
	finParam >> wRow; 
	cout << "  Number of col : " << wCol << "  Number of row : " << wRow << endl;

	finParam >> wSWLON; 
	finParam >> wSWLAT;
	cout << " Sorth West corner Longitude (degree) : " << wSWLON 
		 << " Latitude (deg) : " << wSWLAT << endl;

	const int Col = wCol;     // column number of study area
	const int Row = wRow;     // row number of study area
	const double SWLON = wSWLON; // South west corner of longitude (degree)
	const double SWLAT = wSWLAT; // South west corner of latiitude (degree)

	// ̓f[^~Jf[^̈ input data range
	cout << "2) Reading parameters for input data." << endl;

	int wROFnum_col, wROFnum_row;
	double wDelta_x, wDelta_y, wROF_WLON, wROF_SLAT;

	//int wROFdata_type;	// added by Tanaka T. 2018/6/28
	//finParam >> wROFdata_type;

	finParam >> wROFnum_col;
	finParam >> wROFnum_row;
	cout << "  Number of col : " << wROFnum_col 
		 << "  Number of row : " << wROFnum_row << endl;

	finParam >> wDelta_x;
	finParam >> wDelta_y;
	cout << "  Input data grid size of x-direction : " << wDelta_x << endl;
	cout << "  Input data grid size of y-direction : " << wDelta_y << endl;

	finParam >> wROF_WLON; 
	cout << "  West corner longitude of input data : " << wROF_WLON << endl;
	finParam >> wROF_SLAT; 
	cout << "  South corner latitude of input data : " << wROF_SLAT << endl;
													
	const int ROFnum_col = wROFnum_col;
	const int ROFnum_row = wROFnum_row; 
	const double Delta_x = wDelta_x;
	const double Delta_y = wDelta_y;
	const double ROF_WLON = wROF_WLON;
	const double ROF_SLAT = wROF_SLAT;

	// vZ simulation term
	cout << "3) Reading parameters for simuration duration." << endl;
	int wNdays;
	finParam >> wNdays;
	cout << "  Simulation duration (days) : " << wNdays << endl;

	const int Ndays = wNdays; // Simulation duration (days)

	cout << endl;
	finParam.close();
	//--------------------- end parameter reading ------------------------------------------------

	char DischargeFileName[256];

	// Hourly output data open 
	ifstream FileHourly; // Hourly data
	sprintf_s(DischargeFileName, 256, "./dischargeHourly.bin");
	cout << "Read " << DischargeFileName << endl;
	FileHourly.open(DischargeFileName, ios::in | ios::binary);

	if(!FileHourly){
		cout << "Discharge data cannot open." << endl;
		//cout << "Program ends. Hit any key!" << endl;
		//char ch;
		//cin.get(ch);
		return 1;
	}

	int max_time_counter = Ndays*24;
	if (max_time_counter > 10000) {
		cout << "Max_time_counter exceeds 10000!" << endl;
		return 1;
	}

	float discharge_min, discharge_max, discharge;
	string ss;
	cout << endl;
	cout << "Do you want to set the maximum and minimun values of rainfall intensity? (y/n)" << endl;
	cin >> ss;
	if (ss == "y") {
		cout << "Input maximum value : ";
		cin >> discharge_max;
		cout << "Input minimum value : ";
		cin >> discharge_min;
		if (discharge_min >= discharge_max) {
			cout << "Wrong values inputted!" << endl;
			return 1;
		}
	}
	else {
		discharge_max = -1.0; discharge_min = 10000.0; // Find max and min value of discharge.
		for (int i = 1; i <= max_time_counter; i++) {
			for (int j = 1; j <= Row; j++) {
				for (int k = 1; k <= Col; k++) {
					FileHourly.read((char*)&discharge, sizeof(float));
					if (discharge < 0.0) {
						continue;
					}
					if (discharge > discharge_max) {
						discharge_max = discharge;
					}
					if (discharge < discharge_min) {
						discharge_min = discharge;
					}
				}
			}
		}
	}
	FileHourly.close();

    if (discharge_max <= discharge_min) {
		cout << "Maximun discharge <= Minimum discharge!" << endl;
		return 1;
	}

	cout << "Maximun discharge = " << discharge_max << " Minimum discharge = " << discharge_min << endl;

	// Hourly output data re-open. 
	sprintf_s(DischargeFileName, 256, "./dischargeHourly.bin");
	cout << "Read " << DischargeFileName << endl;
	FileHourly.open(DischargeFileName, ios::in | ios::binary);

	if(!FileHourly){
		cout << "Discharge data cannot open." << endl;
		cout << endl;
		cout << "Program ends. Hit any key!" << endl;
		char ch;
		cin.get(ch);
		return 1;
	}

	int idischarge;
	int** mdischarge;
	mdischarge = new int*[Row];
	for(int i=0; i<Row; i++){
		mdischarge[i] = new int[Col];
	}
	int red, green, blue, pixelCol=5, pixelRow=5;
	double work;

	for (int i = 1; i <= max_time_counter; i++) {

		ofstream ppmFile;
		sprintf_s(DischargeFileName, 256, "./image/%4d_discharge.ppm", i);
		cout << "Write " << DischargeFileName << "\r";

		ppmFile.open(DischargeFileName, ios::out | ios::binary);

		if(!ppmFile){
			cout << "Image output file cannot open." << endl;
			return 1;
		}
		
		ppmFile << "P6" << "\n"; // Write header information.
		ppmFile << Col*pixelCol << " " << Row*pixelRow << "\n";
		ppmFile << "255" << "\n";

		for (int j = 0; j < Row; j++) {
			for (int k = 0; k < Col; k++) {
				FileHourly.read((char*)&discharge, sizeof(float));

				if (j==1 || j==Row-2 || k==1 || k==Col-2) { // out of study area
					mdischarge[j][k] = -999;
				}
				else if (discharge < 0.0) { // out of study area
					mdischarge[j][k] = -999;
				}
				else {
					work=pow((discharge-discharge_min)/(discharge_max-discharge_min), (float)0.5);
					mdischarge[j][k] = (int)(work*255);
				}
			}
		}

		for (int j=Row-1; j >= 0; j--) {
			for (int jx=0; jx < pixelCol; jx++) {
				for (int k = 0; k < Col; k++) {
					idischarge = mdischarge[j][k];
	
					// 256 kind of colors setting.
					setColor(idischarge, red, green, blue);
	
					// each grid corresponds to pixel * pixel.
					for (int kx=0; kx < pixelCol; kx++) {
						ppmFile.put((char)red); // Red
						ppmFile.put((char)green); // Green
						ppmFile.put((char)blue); // Blue
					}
				}
			}
		}
		ppmFile.close();
	}

	cout << endl;
	cout << "Program ends!" << endl;
	//cout << "Program ends. Hit any key!" << endl;
	//char ch;
	//cin.get(ch);
}


void setColor(int input, int& red, int& green, int& blue)
{
	if (input < 0) { // blue
		red = 0;
		green = 0;
		blue = 0;
	}
	else if (0 <= input && input <= 63) { // From blue  to right blue.
		red = 0;
		green = input*4;
		blue = 252;
	}
	else if (64 <= input && input <= 127) { // From light blue to green.
		red = 0;
		green = 252;
		blue = 252 - (input-64)*4;
	}
	else if (128 <= input && input <= 191) { // From green to yellow.
		red = (input - 128)*4;
		green = 252;
		blue = 0;
	}
	else if (192 <= input && input <= 255) { // From yellow to red.
		red = 252;
		green = 252 - (input-192)*4;
		blue = 0;
	}
	else if (255 < input) { // red.
		red = 252;
		green = 252;
		blue = 0;
	}
}
